/****************************************************************************
** Form interface generated from reading ui file './wdgscriptsmanagerbase.ui'
**
** Created: Tue May 20 05:46:47 2008
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef WDGSCRIPTSMANAGERBASE_H
#define WDGSCRIPTSMANAGERBASE_H

#include <qvariant.h>
#include <qpixmap.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class KToolBar;
class KListView;
class QListViewItem;
class KPushButton;
class QFrame;

class WdgScriptsManagerBase : public QWidget
{
    Q_OBJECT

public:
    WdgScriptsManagerBase( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~WdgScriptsManagerBase();

    KListView* scriptsList;
    KToolBar* toolBar;
    KPushButton* btnExec;
    QFrame* line2;
    KPushButton* btnLoad;
    KPushButton* btnUnload;
    QFrame* line3;
    KPushButton* btnInstall;
    KPushButton* btnUninstall;
    QFrame* line4;
    KPushButton* btnNewStuff;

protected:
    QVBoxLayout* WdgScriptsManagerBaseLayout;
    QHBoxLayout* layout1;

protected slots:
    virtual void languageChange();

private:
    QPixmap image0;

};

#endif // WDGSCRIPTSMANAGERBASE_H
