<KrossScripting>
    <ScriptAction
      name="CopyCenter"
      version="1"
      text="Copy Center"
      description="Copy data from one source to another."
      icon="editcopy"
      interpreter="python"
      file="CopyCenter.py" />
</KrossScripting>
