( cd usr/lib ; rm -rf libkchartimageexport.so )
( cd usr/lib ; ln -sf libkchartimageexport.so.4.0.0 libkchartimageexport.so )
( cd usr/lib ; rm -rf libkwmf.so )
( cd usr/lib ; ln -sf libkwmf.so.3.0.0 libkwmf.so )
( cd usr/lib ; rm -rf libkrita_ycbcr_u8.so )
( cd usr/lib ; ln -sf libkrita_ycbcr_u8.so.0.0.0 libkrita_ycbcr_u8.so )
( cd usr/lib ; rm -rf libkrita_lms_f32.so.0 )
( cd usr/lib ; ln -sf libkrita_lms_f32.so.0.0.0 libkrita_lms_f32.so.0 )
( cd usr/lib ; rm -rf libkwordexportfilters.so )
( cd usr/lib ; ln -sf libkwordexportfilters.so.1.0.1 libkwordexportfilters.so )
( cd usr/lib ; rm -rf libkexiguiutils.so.2 )
( cd usr/lib ; ln -sf libkexiguiutils.so.2.0.0 libkexiguiutils.so.2 )
( cd usr/lib ; rm -rf libkformdesigner.so )
( cd usr/lib ; ln -sf libkformdesigner.so.2.0.0 libkformdesigner.so )
( cd usr/lib ; rm -rf libkofficecore.so )
( cd usr/lib ; ln -sf libkofficecore.so.3.0.0 libkofficecore.so )
( cd usr/lib ; rm -rf libkotext.so )
( cd usr/lib ; ln -sf libkotext.so.3.0.0 libkotext.so )
( cd usr/lib ; rm -rf libkowmf.so )
( cd usr/lib ; ln -sf libkowmf.so.2.0.0 libkowmf.so )
( cd usr/lib ; rm -rf libkformdesigner.so.2 )
( cd usr/lib ; ln -sf libkformdesigner.so.2.0.0 libkformdesigner.so.2 )
( cd usr/lib ; rm -rf libkexicore.so )
( cd usr/lib ; ln -sf libkexicore.so.2.0.0 libkexicore.so )
( cd usr/lib ; rm -rf libkdchart.so )
( cd usr/lib ; ln -sf libkdchart.so.0.0.0 libkdchart.so )
( cd usr/lib ; rm -rf libkowmf.so.2 )
( cd usr/lib ; ln -sf libkowmf.so.2.0.0 libkowmf.so.2 )
( cd usr/lib ; rm -rf libkrita_cmyk_u16.so.0 )
( cd usr/lib ; ln -sf libkrita_cmyk_u16.so.0.0.0 libkrita_cmyk_u16.so.0 )
( cd usr/lib ; rm -rf libkexidatatable.so )
( cd usr/lib ; ln -sf libkexidatatable.so.2.0.0 libkexidatatable.so )
( cd usr/lib ; rm -rf libkrita_gray_u16.so.0 )
( cd usr/lib ; ln -sf libkrita_gray_u16.so.0.0.0 libkrita_gray_u16.so.0 )
( cd usr/lib ; rm -rf libkrossapi.so.1 )
( cd usr/lib ; ln -sf libkrossapi.so.1.0.0 libkrossapi.so.1 )
( cd usr/lib ; rm -rf libkrita_ycbcr_u8.so.0 )
( cd usr/lib ; ln -sf libkrita_ycbcr_u8.so.0.0.0 libkrita_ycbcr_u8.so.0 )
( cd usr/lib ; rm -rf libkritacommon.so.1 )
( cd usr/lib ; ln -sf libkritacommon.so.1.0.0 libkritacommon.so.1 )
( cd usr/lib ; rm -rf libkopalette.so.1 )
( cd usr/lib ; ln -sf libkopalette.so.1.0.0 libkopalette.so.1 )
( cd usr/lib ; rm -rf libkritascripting.so )
( cd usr/lib ; ln -sf libkritascripting.so.0.0.0 libkritascripting.so )
( cd usr/lib ; rm -rf libkoproperty.so.2 )
( cd usr/lib ; ln -sf libkoproperty.so.2.0.0 libkoproperty.so.2 )
( cd usr/lib ; rm -rf libkritargb.so )
( cd usr/lib ; ln -sf libkritargb.so.0.0.0 libkritargb.so )
( cd usr/lib ; rm -rf libkopainter.so )
( cd usr/lib ; ln -sf libkopainter.so.2.0.0 libkopainter.so )
( cd usr/lib ; rm -rf libkrita_ycbcr_u16.so )
( cd usr/lib ; ln -sf libkrita_ycbcr_u16.so.0.0.0 libkrita_ycbcr_u16.so )
( cd usr/lib ; rm -rf libkrita_rgb_u16.so.0 )
( cd usr/lib ; ln -sf libkrita_rgb_u16.so.0.0.0 libkrita_rgb_u16.so.0 )
( cd usr/lib ; rm -rf libkwmailmerge_interface.so.4 )
( cd usr/lib ; ln -sf libkwmailmerge_interface.so.4.0.0 libkwmailmerge_interface.so.4 )
( cd usr/lib ; rm -rf libkrita_ycbcr_u16.so.0 )
( cd usr/lib ; ln -sf libkrita_ycbcr_u16.so.0.0.0 libkrita_ycbcr_u16.so.0 )
( cd usr/lib ; rm -rf libkexiextendedwidgets.so )
( cd usr/lib ; ln -sf libkexiextendedwidgets.so.2.0.0 libkexiextendedwidgets.so )
( cd usr/lib ; rm -rf libkarboncommon.so.0 )
( cd usr/lib ; ln -sf libkarboncommon.so.0.0.0 libkarboncommon.so.0 )
( cd usr/lib ; rm -rf libkspreadcommon.so )
( cd usr/lib ; ln -sf libkspreadcommon.so.0.0.0 libkspreadcommon.so )
( cd usr/lib ; rm -rf libkarboncommon.so )
( cd usr/lib ; ln -sf libkarboncommon.so.0.0.0 libkarboncommon.so )
( cd usr/lib ; rm -rf libkritacolor.so.1 )
( cd usr/lib ; ln -sf libkritacolor.so.1.0.0 libkritacolor.so.1 )
( cd usr/lib ; rm -rf libkexiextendedwidgets.so.2 )
( cd usr/lib ; ln -sf libkexiextendedwidgets.so.2.0.0 libkexiextendedwidgets.so.2 )
( cd usr/lib ; rm -rf libkiviocommon.so.0 )
( cd usr/lib ; ln -sf libkiviocommon.so.0.0.0 libkiviocommon.so.0 )
( cd usr/lib ; rm -rf libkochart.so )
( cd usr/lib ; ln -sf libkochart.so.1.0.0 libkochart.so )
( cd usr/lib ; rm -rf libkeximain.so )
( cd usr/lib ; ln -sf libkeximain.so.2.0.0 libkeximain.so )
( cd usr/lib ; rm -rf libkexiutils.so )
( cd usr/lib ; ln -sf libkexiutils.so.2.0.0 libkexiutils.so )
( cd usr/lib ; rm -rf libkritaui.so.1 )
( cd usr/lib ; ln -sf libkritaui.so.1.0.0 libkritaui.so.1 )
( cd usr/lib ; rm -rf libkritagrayscale.so )
( cd usr/lib ; ln -sf libkritagrayscale.so.0.0.0 libkritagrayscale.so )
( cd usr/lib ; rm -rf libkexidbparser.so )
( cd usr/lib ; ln -sf libkexidbparser.so.2.0.0 libkexidbparser.so )
( cd usr/lib ; rm -rf libkrossapi.so )
( cd usr/lib ; ln -sf libkrossapi.so.1.0.0 libkrossapi.so )
( cd usr/lib ; rm -rf libkexidatatable.so.2 )
( cd usr/lib ; ln -sf libkexidatatable.so.2.0.0 libkexidatatable.so.2 )
( cd usr/lib ; rm -rf libkrita_rgb_f32.so.0 )
( cd usr/lib ; ln -sf libkrita_rgb_f32.so.0.0.0 libkrita_rgb_f32.so.0 )
( cd usr/lib ; rm -rf libkpresenterprivate.so.4 )
( cd usr/lib ; ln -sf libkpresenterprivate.so.4.0.0 libkpresenterprivate.so.4 )
( cd usr/lib ; rm -rf libkspreadcommon.so.0 )
( cd usr/lib ; ln -sf libkspreadcommon.so.0.0.0 libkspreadcommon.so.0 )
( cd usr/lib ; rm -rf libkwmailmerge_interface.so )
( cd usr/lib ; ln -sf libkwmailmerge_interface.so.4.0.0 libkwmailmerge_interface.so )
( cd usr/lib ; rm -rf libkexisql2.so )
( cd usr/lib ; ln -sf libkexisql2.so.2.0.8 libkexisql2.so )
( cd usr/lib ; rm -rf libkexisql3.so )
( cd usr/lib ; ln -sf libkexisql3.so.3.0.2 libkexisql3.so )
( cd usr/lib ; rm -rf libkexiformutils.so )
( cd usr/lib ; ln -sf libkexiformutils.so.2.0.0 libkexiformutils.so )
( cd usr/lib ; rm -rf libkpresenterimageexport.so )
( cd usr/lib ; ln -sf libkpresenterimageexport.so.4.0.0 libkpresenterimageexport.so )
( cd usr/lib ; rm -rf libkritargb.so.0 )
( cd usr/lib ; ln -sf libkritargb.so.0.0.0 libkritargb.so.0 )
( cd usr/lib ; rm -rf libkexisql2.so.2 )
( cd usr/lib ; ln -sf libkexisql2.so.2.0.8 libkexisql2.so.2 )
( cd usr/lib ; rm -rf libkexisql3.so.3 )
( cd usr/lib ; ln -sf libkexisql3.so.3.0.2 libkexisql3.so.3 )
( cd usr/lib ; rm -rf libkchartimageexport.so.4 )
( cd usr/lib ; ln -sf libkchartimageexport.so.4.0.0 libkchartimageexport.so.4 )
( cd usr/lib ; rm -rf libkritaui.so )
( cd usr/lib ; ln -sf libkritaui.so.1.0.0 libkritaui.so )
( cd usr/lib ; rm -rf libkdchart.so.0 )
( cd usr/lib ; ln -sf libkdchart.so.0.0.0 libkdchart.so.0 )
( cd usr/lib ; rm -rf libkchartcommon.so.2 )
( cd usr/lib ; ln -sf libkchartcommon.so.2.0.0 libkchartcommon.so.2 )
( cd usr/lib ; rm -rf libkopainter.so.2 )
( cd usr/lib ; ln -sf libkopainter.so.2.0.0 libkopainter.so.2 )
( cd usr/lib ; rm -rf libkofficeui.so.3 )
( cd usr/lib ; ln -sf libkofficeui.so.3.0.0 libkofficeui.so.3 )
( cd usr/lib ; rm -rf libkexiformutils.so.2 )
( cd usr/lib ; ln -sf libkexiformutils.so.2.0.0 libkexiformutils.so.2 )
( cd usr/lib ; rm -rf libkexicore.so.2 )
( cd usr/lib ; ln -sf libkexicore.so.2.0.0 libkexicore.so.2 )
( cd usr/lib ; rm -rf libkofficecore.so.3 )
( cd usr/lib ; ln -sf libkofficecore.so.3.0.0 libkofficecore.so.3 )
( cd usr/lib ; rm -rf libkexiguiutils.so )
( cd usr/lib ; ln -sf libkexiguiutils.so.2.0.0 libkexiguiutils.so )
( cd usr/lib ; rm -rf libkexidbparser.so.2 )
( cd usr/lib ; ln -sf libkexidbparser.so.2.0.0 libkexidbparser.so.2 )
( cd usr/lib ; rm -rf libkexirelationsview.so.2 )
( cd usr/lib ; ln -sf libkexirelationsview.so.2.0.0 libkexirelationsview.so.2 )
( cd usr/lib ; rm -rf libkochart.so.1 )
( cd usr/lib ; ln -sf libkochart.so.1.0.0 libkochart.so.1 )
( cd usr/lib ; rm -rf libkritascripting.so.0 )
( cd usr/lib ; ln -sf libkritascripting.so.0.0.0 libkritascripting.so.0 )
( cd usr/lib ; rm -rf libkritacommon.so )
( cd usr/lib ; ln -sf libkritacommon.so.1.0.0 libkritacommon.so )
( cd usr/lib ; rm -rf libkstore.so )
( cd usr/lib ; ln -sf libkstore.so.3.0.0 libkstore.so )
( cd usr/lib ; rm -rf libkeximain.so.2 )
( cd usr/lib ; ln -sf libkeximain.so.2.0.0 libkeximain.so.2 )
( cd usr/lib ; rm -rf libkofficeui.so )
( cd usr/lib ; ln -sf libkofficeui.so.3.0.0 libkofficeui.so )
( cd usr/lib ; rm -rf libkexidb.so )
( cd usr/lib ; ln -sf libkexidb.so.2.0.0 libkexidb.so )
( cd usr/lib ; rm -rf libkopalette.so )
( cd usr/lib ; ln -sf libkopalette.so.1.0.0 libkopalette.so )
( cd usr/lib ; rm -rf libkformulalib.so )
( cd usr/lib ; ln -sf libkformulalib.so.4.0.0 libkformulalib.so )
( cd usr/lib ; rm -rf libkformulalib.so.4 )
( cd usr/lib ; ln -sf libkformulalib.so.4.0.0 libkformulalib.so.4 )
( cd usr/lib ; rm -rf libkwordprivate.so )
( cd usr/lib ; ln -sf libkwordprivate.so.4.0.0 libkwordprivate.so )
( cd usr/lib ; rm -rf libkritagrayscale.so.0 )
( cd usr/lib ; ln -sf libkritagrayscale.so.0.0.0 libkritagrayscale.so.0 )
( cd usr/lib ; rm -rf libkrita_rgb_f32.so )
( cd usr/lib ; ln -sf libkrita_rgb_f32.so.0.0.0 libkrita_rgb_f32.so )
( cd usr/lib ; rm -rf libkwordprivate.so.4 )
( cd usr/lib ; ln -sf libkwordprivate.so.4.0.0 libkwordprivate.so.4 )
( cd usr/lib ; rm -rf libkexiutils.so.2 )
( cd usr/lib ; ln -sf libkexiutils.so.2.0.0 libkexiutils.so.2 )
( cd usr/lib ; rm -rf libkpresenterimageexport.so.4 )
( cd usr/lib ; ln -sf libkpresenterimageexport.so.4.0.0 libkpresenterimageexport.so.4 )
( cd usr/lib ; rm -rf libkugarlib.so )
( cd usr/lib ; ln -sf libkugarlib.so.1.0.2 libkugarlib.so )
( cd usr/lib ; rm -rf libkeximigrate.so.2 )
( cd usr/lib ; ln -sf libkeximigrate.so.2.0.0 libkeximigrate.so.2 )
( cd usr/lib ; rm -rf libkritaimage.so.1 )
( cd usr/lib ; ln -sf libkritaimage.so.1.0.0 libkritaimage.so.1 )
( cd usr/lib ; rm -rf libkritacolor.so )
( cd usr/lib ; ln -sf libkritacolor.so.1.0.0 libkritacolor.so )
( cd usr/lib ; rm -rf libkrossmain.so.1 )
( cd usr/lib ; ln -sf libkrossmain.so.1.0.0 libkrossmain.so.1 )
( cd usr/lib ; rm -rf libkrita_cmyk_u16.so )
( cd usr/lib ; ln -sf libkrita_cmyk_u16.so.0.0.0 libkrita_cmyk_u16.so )
( cd usr/lib ; rm -rf libkrita_gray_u16.so )
( cd usr/lib ; ln -sf libkrita_gray_u16.so.0.0.0 libkrita_gray_u16.so )
( cd usr/lib ; rm -rf libkrita_lms_f32.so )
( cd usr/lib ; ln -sf libkrita_lms_f32.so.0.0.0 libkrita_lms_f32.so )
( cd usr/lib ; rm -rf libkwmf.so.3 )
( cd usr/lib ; ln -sf libkwmf.so.3.0.0 libkwmf.so.3 )
( cd usr/lib ; rm -rf libkeximigrate.so )
( cd usr/lib ; ln -sf libkeximigrate.so.2.0.0 libkeximigrate.so )
( cd usr/lib ; rm -rf libkwordexportfilters.so.1 )
( cd usr/lib ; ln -sf libkwordexportfilters.so.1.0.1 libkwordexportfilters.so.1 )
( cd usr/lib ; rm -rf libkugarlib.so.1 )
( cd usr/lib ; ln -sf libkugarlib.so.1.0.2 libkugarlib.so.1 )
( cd usr/lib ; rm -rf libkiviocommon.so )
( cd usr/lib ; ln -sf libkiviocommon.so.0.0.0 libkiviocommon.so )
( cd usr/lib ; rm -rf libkritaimage.so )
( cd usr/lib ; ln -sf libkritaimage.so.1.0.0 libkritaimage.so )
( cd usr/lib ; rm -rf libkrita_rgb_u16.so )
( cd usr/lib ; ln -sf libkrita_rgb_u16.so.0.0.0 libkrita_rgb_u16.so )
( cd usr/lib ; rm -rf libkrossmain.so )
( cd usr/lib ; ln -sf libkrossmain.so.1.0.0 libkrossmain.so )
( cd usr/lib ; rm -rf libkstore.so.3 )
( cd usr/lib ; ln -sf libkstore.so.3.0.0 libkstore.so.3 )
( cd usr/lib ; rm -rf libkchartcommon.so )
( cd usr/lib ; ln -sf libkchartcommon.so.2.0.0 libkchartcommon.so )
( cd usr/lib ; rm -rf libkotext.so.3 )
( cd usr/lib ; ln -sf libkotext.so.3.0.0 libkotext.so.3 )
( cd usr/lib ; rm -rf libkoproperty.so )
( cd usr/lib ; ln -sf libkoproperty.so.2.0.0 libkoproperty.so )
( cd usr/lib ; rm -rf libkexidb.so.2 )
( cd usr/lib ; ln -sf libkexidb.so.2.0.0 libkexidb.so.2 )
( cd usr/lib ; rm -rf libkpresenterprivate.so )
( cd usr/lib ; ln -sf libkpresenterprivate.so.4.0.0 libkpresenterprivate.so )
( cd usr/lib ; rm -rf libkexirelationsview.so )
( cd usr/lib ; ln -sf libkexirelationsview.so.2.0.0 libkexirelationsview.so )
( cd usr/share/doc/HTML/en/kexi ; rm -rf common )
( cd usr/share/doc/HTML/en/kexi ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kivio ; rm -rf common )
( cd usr/share/doc/HTML/en/kivio ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/krita ; rm -rf common )
( cd usr/share/doc/HTML/en/krita ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kugar ; rm -rf common )
( cd usr/share/doc/HTML/en/kugar ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kword ; rm -rf common )
( cd usr/share/doc/HTML/en/kword ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/koffice ; rm -rf common )
( cd usr/share/doc/HTML/en/koffice ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/koshell ; rm -rf common )
( cd usr/share/doc/HTML/en/koshell ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kspread ; rm -rf common )
( cd usr/share/doc/HTML/en/kspread ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/karbon ; rm -rf common )
( cd usr/share/doc/HTML/en/karbon ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kchart ; rm -rf common )
( cd usr/share/doc/HTML/en/kchart ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kpresenter ; rm -rf common )
( cd usr/share/doc/HTML/en/kpresenter ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kplato ; rm -rf common )
( cd usr/share/doc/HTML/en/kplato ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/thesaurus ; rm -rf common )
( cd usr/share/doc/HTML/en/thesaurus ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/doc/HTML/en/kformula ; rm -rf common )
( cd usr/share/doc/HTML/en/kformula ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/share/apps/kivio/autoloadStencils ; rm -rf straight_connector.so )
( cd usr/share/apps/kivio/autoloadStencils ; ln -sf /usr/lib/kde3/straight_connector.so straight_connector.so )
