<KrossScripting>
    <ScriptAction
      name="ChangeColorspaceRuby"
      text="Change Colorspace"
      description="Demonstrate how to change the colorspace in a script"
      icon=""
      interpreter="ruby"
      file="changecs.rb" />
</KrossScripting>
