/****************************************************************************
** Form interface generated from reading ui file './KoGeneralPropertyUi.ui'
**
** Created: Tue May 20 05:40:38 2008
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef KOGENERALPROPERTYUI_H
#define KOGENERALPROPERTYUI_H

#include <qvariant.h>
#include <qpixmap.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class KoUnitDoubleSpinBox;
class QLabel;
class QLineEdit;
class QCheckBox;
class QGroupBox;

class KoGeneralPropertyUI : public QWidget
{
    Q_OBJECT

public:
    KoGeneralPropertyUI( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~KoGeneralPropertyUI();

    QLabel* nameLabel;
    QLineEdit* nameInput;
    QCheckBox* protect;
    QCheckBox* keepRatio;
    QGroupBox* positionGroup;
    QLabel* widthLabel;
    QLabel* heightLabel;
    QLabel* topLabel;
    QLabel* leftLabel;
    KoUnitDoubleSpinBox* xInput;
    KoUnitDoubleSpinBox* yInput;
    KoUnitDoubleSpinBox* heightInput;
    KoUnitDoubleSpinBox* widthInput;

protected:
    QGridLayout* KoGeneralPropertyUILayout;
    QSpacerItem* spacer1;
    QHBoxLayout* layout13;
    QGridLayout* positionGroupLayout;

protected slots:
    virtual void languageChange();

private:
    QPixmap image0;

};

#endif // KOGENERALPROPERTYUI_H
