<KrossScripting>
    <ScriptAction
      name="FiltersTestRuby"
      text="Filters Test"
      description="Test of filters in scripting (ruby)"
      icon=""
      interpreter="ruby"
      file="filterstest.rb" />
</KrossScripting>
